# -*- coding: utf-8 -*-

import re
from os.path import join
from framework import LinguistTestBase, main, ROOT_DIR
from libs.tokenizer import Tokenizer


class TestTokenizer(LinguistTestBase):
    
    def tokenize(self, data='', is_path=None):
        if is_path: 
            data = open(join(join(ROOT_DIR, "samples"), str(data))).read()
        return Tokenizer.tokenize(data)

    def test_skip_string_literals(self):
        r = ["print"]
        assert r == self.tokenize('print ""')
        assert r == self.tokenize('print "Josh"')
        assert r == self.tokenize("print 'Josh'")
        assert r == self.tokenize('print "Hello \\"Josh\\""')
        assert r == self.tokenize("print 'Hello \\'Josh\\''")
        assert r == self.tokenize("print \"Hello\", \"Josh\"")
        assert r == self.tokenize("print 'Hello', 'Josh'")
        assert r == self.tokenize("print \"Hello\", \"\", \"Josh\"")
        assert r == self.tokenize("print 'Hello', '', 'Josh'")

    def test_skip_number_literals(self):
        assert ['+'] == self.tokenize('1 + 1')
        assert ['add', '(', ')'] == self.tokenize('add(123, 456)')
        assert ['|'] == self.tokenize('0x01 | 0x10')
        assert ['*'] == self.tokenize('500.42 * 1.0')

    def test_skip_comments(self):
        r1, r2, r3 = ['foo'], ['foo', 'bar'], ['%']
        assert r1 == self.tokenize("foo\n# Comment")
        assert r1 == self.tokenize("foo\n# Comment")
        assert r2 == self.tokenize("foo\n# Comment\nbar")
        assert r1 == self.tokenize("foo\n// Comment")
        assert r1 == self.tokenize("foo /* Comment */")
        assert r1 == self.tokenize("foo /* \nComment \n */")
        assert r1 == self.tokenize("foo <!-- Comment -->")
        assert r1 == self.tokenize("foo {- Comment -}")
        assert r1 == self.tokenize("foo (* Comment *)")
        assert r3 == self.tokenize("2 % 10\n% Comment")

    def test_sgml_tags(self):
        assert ["<html>","</html>"] == self.tokenize("<html> </html>")
        assert ["<div>","id","</div>"] == self.tokenize("<div id></div>")
        assert ["<div>","id=","</div>"] == self.tokenize("<div id=foo></div>")
        assert ["<div>","id","class","</div>"] == self.tokenize("<div id class></div>")
        assert ["<div>","id=","</div>"] == self.tokenize("<div id=\"foo bar\"></div>")
        assert ["<div>","id=","</div>"] == self.tokenize("<div id='foo bar'></div>")
        assert ["<?xml>","version="] == self.tokenize("<?xml version=\"1.0\"?>")

    def test_operators(self):
      assert ["+"] == self.tokenize("1 + 1")
      assert ["-"] == self.tokenize("1 - 1")
      assert ["*"] == self.tokenize("1 * 1")
      assert ["/"] == self.tokenize("1 / 1")
      assert ["%"] == self.tokenize("2 % 5")
      assert ["&"] == self.tokenize("1 & 1")
      assert ["&&"] == self.tokenize("1 && 1")
      assert ["|"] == self.tokenize("1 | 1")
      assert ["||"] == self.tokenize("1 || 1")
      assert ["<"] == self.tokenize("1 < 0x01")
      assert ["<<"] == self.tokenize("1 << 0x01")

    def test_c_tokens(self):
        r1 = "#ifndef HELLO_H #define HELLO_H void hello ( ) ; #endif".split()
        assert r1 == self.tokenize("C/hello.h", True)
        r2 = "#include <stdio.h> int main ( ) { printf ( ) ; return ; }".split()
        assert r2 == self.tokenize("C/hello.c", True)

    def test_cpp_tokens(self):
        r1 = "class Bar { protected char *name ; public void hello ( ) ; }".split()
        assert r1 == self.tokenize("C++/bar.h", True)
        r2 = "#include <iostream> using namespace std ; int main ( ) { cout << << endl ; }".split()
        assert r2 == self.tokenize("C++/hello.cpp", True)

    def test_objective_c_tokens(self):
        r1 = "#import <Foundation/Foundation.h> @interface Foo NSObject { } @end".split()
        assert r1 == self.tokenize("Objective-C/Foo.h", True)
        r2 = "#import <Cocoa/Cocoa.h> int main ( int argc char *argv [ ] ) { NSLog ( @ ) ; return ; }".split() 
        assert r2 == self.tokenize("Objective-C/hello.m", True)
        assert "#import @implementation Foo @end".split() == self.tokenize("Objective-C/Foo.m", True)
        
    def test_shebang(self):
        assert "SHEBANG#!sh" == self.tokenize("Shell/sh.script!",True)[0]
        assert "SHEBANG#!bash" == self.tokenize("Shell/bash.script!",True)[0]
        assert "SHEBANG#!zsh" == self.tokenize("Shell/zsh.script!",True)[0]
        assert "SHEBANG#!perl" == self.tokenize("Perl/perl.script!",True)[0]
        assert "SHEBANG#!python" == self.tokenize("Python/python.script!",True)[0]
        assert "SHEBANG#!ruby" == self.tokenize("Ruby/ruby.script!",True)[0]
        assert "SHEBANG#!ruby" == self.tokenize("Ruby/ruby2.script!",True)[0]
        assert "SHEBANG#!node" == self.tokenize("JavaScript/js.script!",True)[0]
        assert "SHEBANG#!php" == self.tokenize("PHP/php.script!",True)[0]
        assert "echo" == self.tokenize("Shell/invalid-shebang.sh",True)[0]

    def test_javscript_tokens(self):
        r = ["(", "function", "(", ")", "{", "console.log", "(", ")", ";", "}", ")", 
             ".call", "(", "this", ")", ";"]  
        assert r == self.tokenize("JavaScript/hello.js", True)

    def test_json_tokens(self):
        assert "{ [ ] { } }".split() == self.tokenize("JSON/product.json", True)
        
    def test_ruby_tokens(self):
        assert "module Foo end".split() == self.tokenize("Ruby/foo.rb", True)
        assert "task default do puts end".split(), self.tokenize("Ruby/filenames/Rakefile", True)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

import re

class StringScanner(object):

    def __init__(self, string, dup=False):
        self.string = string
        self.pos_history, self._pos = [0], 0
        self.match_history, self._match = [None], None

    def __repr__(self):
        return '<StringScanner pos: %s, match: %s>' % (self.pos, self.match)

    @property
    def is_eos(self):
        """
        Return true if the scan pointer is at the end of the string. 
        """
        return self.pos >= len(self.string)

    @property
    def getch(self):
        """
        Get a single character and advance the scan pointer.
        """
        self.pos += 1
        return self.string[self.pos - 1:self.pos]

    def peek(self, length):
        """
        Get a number of characters without advancing the scan pointer.
        """
        return self.string[self.pos:self.pos + length]

    @property
    def rest(self):
        """
        Return true if the scan pointer is at the end of the string. 
        """
        return self.string[self.pos:]

    @property
    def coords(self):
        """
        Return the current scanner position as `(lineno, columnno, line)`.

        This method is useful for displaying the scanner position in a human-
        readable way. For example, you could use it to provide friendlier
        debugging information when writing parsers.
        """
        return text_coords(self.string, self.pos)

    @property
    def is_tainted(self):
        return False

    @property
    def pos(self):
        return self._pos

    @pos.setter
    def pos(self, pos):
        # The current position of the scan pointer.
        self._pos = pos
        self.pos_history.append(pos)

    @property
    def prev(self):
        # The last position of the scan pointer.
        return self.pos_history[-2]
        
    @property
    def match(self):
        return self._match

    @match.setter
    def match(self, match):
        # The latest scan match.
        self._match = match
        self.match_history.append(match)

    @property
    def matched(self):
        """
        Get the whole of the current match.
        This method returns whatever would have been returned by the latest
        :meth:`scan()` call.
        """
        return self.match.group(0)

    @property
    def pre_match(self):
        """
        Get whatever comes before the current match.
        """
        return self.string[:self.match.start()]

    @property
    def post_match(self):
        """
        Get whatever comes after the current match. 
        """
        return self.string[self.match.end():]

    @property
    def unscan(self):
        """
        Undo the last scan, resetting the position and match registers.
        """
        self.pos_history.pop()
        self._pos = self.pos_history[-1]
        self.match_history.pop()
        self._match = self.match_history[-1]

    @property
    def is_beginning_of_line(self):
        """ 
        Return true if the scan pointer is at the beginning of a line.
        """
        if self.pos > len(self.string):
            return False
        elif self.pos == 0:
            return True
        return self.string[self.pos - 1] == "\n"

    @property
    def terminate(self):
        """
        Set the scan pointer to the end of the string; 
        clear match data.
        """
        self.pos = len(self.string)
        self.match = None

    def scan_full(self, regex, return_string=True, advance_pointer=True):
        """
        Match from the current position.

        If `return_string` is false and a match is found, returns the number of
        characters matched.
        """
        regex = get_regex(regex)
        self.match = regex.match(self.string, self.pos)
        if not self.match:
            return
        if advance_pointer:
            self.pos = self.match.end()
        if return_string:
            return self.match.group(0)
        return len(self.match.group(0))

    def search_full(self, regex, return_string=True, advance_pointer=True):
        """
        Search from the current position.

        If `return_string` is false and a match is found, returns the number of
        characters matched (from the current position *up to* the end of the
        match).
        """
        regex = get_regex(regex)
        self.match = regex.search(self.string, self.pos)
        if not self.match:
            return
        start_pos = self.pos
        if advance_pointer:
            self.pos = self.match.end()
        if return_string:
            return self.string[start_pos:self.match.end()]
        return (self.match.end() - start_pos)


    def scan(self, regex):
        """
        Match a pattern from the current position.

        If a match is found, advances the scan pointer and returns the matched
        string. Otherwise returns ``None``.
        """
        return self.scan_full(regex, return_string=True, advance_pointer=True)

    def scan_until(self, regex):
        """
        Search for a pattern from the current position.

        If a match is found, advances the scan pointer and returns the matched
        string, from the current position *up to* the end of the match.
        Otherwise returns ``None``.
        """
        return self.search_full(regex, return_string=True, advance_pointer=True)

    def scan_upto(self, regex):
        """
        Scan up to, but not including, the given regex. 
        """
        pos = self.pos
        if self.scan_until(regex) is not None:
            self.pos -= len(self.matched)
            # Remove the intermediate position history entry.
            self.pos_history.pop(-2)
            return self.pre_match[pos:]

    def skip(self, regex):
        """
        Like :meth:`scan`, but return the number of characters matched.
        """
        return self.scan_full(regex, return_string=False, advance_pointer=True)

    def skip_until(self, regex):
        """
        Like :meth:`scan_until`, but return the number of characters matched.
        """
        return self.search_full(regex, return_string=False, advance_pointer=True)

    def check(self, regex):
        """
        See what :meth:`scan` would return without advancing the pointer.
        """
        return self.scan_full(regex, return_string=True, advance_pointer=False)

    def check_until(self, regex):
        """
        See what :meth:`scan_until` would return without advancing the pointer.
        """
        return self.search_full(regex, return_string=True, advance_pointer=False)

    def exists(self, regex):
        """
        See what :meth:`skip_until` would return without advancing the pointer.
        Returns the number of characters matched if it does exist, or ``None``
        otherwise.
        """
        return self.search_full(regex, return_string=False, advance_pointer=False)


def text_coords(string, position):
    """
    Transform a simple index into a human-readable position in a string.

    This function accepts a string and an index, and will return a triple 
    of `(lineno, columnno, line)` representing the position through the 
    text. 
    
    It's useful for displaying a string index in a human-readable way.
    """
    line_start = string.rfind('\n', 0, position) + 1
    line_end = string.find('\n', position)
    lineno = string.count('\n', 0, position)
    columnno = position - line_start
    line = string[line_start:line_end]
    return (lineno, columnno, line)


def get_regex(regex):
    """
    Ensure we have a compiled regular expression object.
    """
    if isinstance(regex, basestring):
        return re.compile(regex)
    elif not isinstance(regex, re._pattern_type):
        raise TypeError("Invalid regex type: %r" % (regex,))
    return regex



# -*- coding: utf-8 -*-

from pygments.lexers import find_lexer_class
from framework import LinguistTestBase, main
from libs.language import Language


colorize = """<div class="highlight"><pre><span class="k">def</span> <span class="nf">foo</span>
  <span class="s1">&#39;foo&#39;</span>
<span class="k">end</span>
</pre></div>
"""

class TestLanguage(LinguistTestBase):

    def test_lexer(self):
        assert find_lexer_class('ActionScript 3') == Language.find_by_name('ActionScript').lexer
        assert find_lexer_class('ActionScript 3') == Language.find_by_name('ActionScript').lexer
        assert find_lexer_class('Bash') == Language.find_by_name('Gentoo Ebuild').lexer
        assert find_lexer_class('Bash') == Language.find_by_name('Gentoo Eclass').lexer
        assert find_lexer_class('Bash') == Language.find_by_name('Shell').lexer
        assert find_lexer_class('C') == Language.find_by_name('OpenCL').lexer
        assert find_lexer_class('C') == Language.find_by_name('XS').lexer
        assert find_lexer_class('C++') == Language.find_by_name('C++').lexer
        assert find_lexer_class('Coldfusion HTML') == Language.find_by_name('ColdFusion').lexer
        assert find_lexer_class('Coq') == Language.find_by_name('Coq').lexer
        assert find_lexer_class('FSharp') == Language.find_by_name('F#').lexer
        assert find_lexer_class('FSharp') == Language.find_by_name('F#').lexer
        assert find_lexer_class('Fortran') == Language.find_by_name('FORTRAN').lexer
        assert find_lexer_class('Gherkin') == Language.find_by_name('Cucumber').lexer
        assert find_lexer_class('Groovy') == Language.find_by_name('Groovy').lexer
        assert find_lexer_class('HTML') == Language.find_by_name('HTML').lexer
        assert find_lexer_class('HTML+Django/Jinja') == Language.find_by_name('HTML+Django').lexer
        assert find_lexer_class('HTML+PHP') == Language.find_by_name('HTML+PHP').lexer
        assert find_lexer_class('HTTP') == Language.find_by_name('HTTP').lexer
        assert find_lexer_class('JSON') == Language.find_by_name('JSON').lexer
        assert find_lexer_class('Java') == Language.find_by_name('ChucK').lexer
        assert find_lexer_class('Java') == Language.find_by_name('Java').lexer
        assert find_lexer_class('JavaScript') == Language.find_by_name('JavaScript').lexer
        assert find_lexer_class('MOOCode') == Language.find_by_name('Moocode').lexer
        assert find_lexer_class('MuPAD') == Language.find_by_name('mupad').lexer
        assert find_lexer_class('NASM') == Language.find_by_name('Assembly').lexer
        assert find_lexer_class('OCaml') == Language.find_by_name('OCaml').lexer
        assert find_lexer_class('Ooc') == Language.find_by_name('ooc').lexer
        assert find_lexer_class('OpenEdge ABL') == Language.find_by_name('OpenEdge ABL').lexer
        assert find_lexer_class('REBOL') == Language.find_by_name('Rebol').lexer
        assert find_lexer_class('RHTML') == Language.find_by_name('HTML+ERB').lexer
        assert find_lexer_class('RHTML') == Language.find_by_name('RHTML').lexer
        assert find_lexer_class('Ruby') == Language.find_by_name('Mirah').lexer
        assert find_lexer_class('Ruby') == Language.find_by_name('Ruby').lexer
        assert find_lexer_class('S') == Language.find_by_name('R').lexer
        assert find_lexer_class('Scheme') == Language.find_by_name('Emacs Lisp').lexer
        assert find_lexer_class('Scheme') == Language.find_by_name('Nu').lexer
        assert find_lexer_class('Racket') == Language.find_by_name('Racket').lexer
        assert find_lexer_class('Scheme') == Language.find_by_name('Scheme').lexer
        assert find_lexer_class('Standard ML') == Language.find_by_name('Standard ML').lexer
        assert find_lexer_class('TeX') == Language.find_by_name('TeX').lexer
        #FIXME bug by pygments
        # assert find_lexer_class('Verilog') == Language.find_by_name('Verilog').lexer
        assert find_lexer_class('XSLT') == Language.find_by_name('XSLT').lexer
        assert find_lexer_class('aspx-vb') == Language.find_by_name('ASP').lexer
        assert find_lexer_class('haXe') == Language.find_by_name('Haxe').lexer
        assert find_lexer_class('reStructuredText') == Language.find_by_name('reStructuredText').lexer


    def test_find_by_alias(self):
        assert Language.find_by_name('ASP') == Language.find_by_alias('asp')
        assert Language.find_by_name('ASP') == Language.find_by_alias('aspx')
        assert Language.find_by_name('ASP') == Language.find_by_alias('aspx-vb')
        assert Language.find_by_name('ActionScript') == Language.find_by_alias('as3')
        assert Language.find_by_name('ApacheConf') == Language.find_by_alias('apache')
        assert Language.find_by_name('Assembly') == Language.find_by_alias('nasm')
        assert Language.find_by_name('Batchfile') == Language.find_by_alias('bat')
        assert Language.find_by_name('C#') == Language.find_by_alias('c#')
        assert Language.find_by_name('C#') == Language.find_by_alias('csharp')
        assert Language.find_by_name('C') == Language.find_by_alias('c')
        assert Language.find_by_name('C++') == Language.find_by_alias('c++')
        assert Language.find_by_name('C++') == Language.find_by_alias('cpp')
        assert Language.find_by_name('CoffeeScript') == Language.find_by_alias('coffee')
        assert Language.find_by_name('CoffeeScript') == Language.find_by_alias('coffee-script')
        assert Language.find_by_name('ColdFusion') == Language.find_by_alias('cfm')
        assert Language.find_by_name('Common Lisp') == Language.find_by_alias('common-lisp')
        assert Language.find_by_name('Common Lisp') == Language.find_by_alias('lisp')
        assert Language.find_by_name('Darcs Patch') == Language.find_by_alias('dpatch')
        assert Language.find_by_name('Dart') == Language.find_by_alias('dart')
        assert Language.find_by_name('Emacs Lisp') == Language.find_by_alias('elisp')
        assert Language.find_by_name('Emacs Lisp') == Language.find_by_alias('emacs')
        assert Language.find_by_name('Emacs Lisp') == Language.find_by_alias('emacs-lisp')
        assert Language.find_by_name('Gettext Catalog') == Language.find_by_alias('pot')
        assert Language.find_by_name('HTML') == Language.find_by_alias('html')
        assert Language.find_by_name('HTML') == Language.find_by_alias('xhtml')
        assert Language.find_by_name('HTML+ERB') == Language.find_by_alias('html+erb')
        assert Language.find_by_name('HTML+ERB') == Language.find_by_alias('erb')
        assert Language.find_by_name('IRC log') == Language.find_by_alias('irc')
        assert Language.find_by_name('JSON') == Language.find_by_alias('json')
        assert Language.find_by_name('Java Server Pages') == Language.find_by_alias('jsp')
        assert Language.find_by_name('Java') == Language.find_by_alias('java')
        assert Language.find_by_name('JavaScript') == Language.find_by_alias('javascript')
        assert Language.find_by_name('JavaScript') == Language.find_by_alias('js')
        assert Language.find_by_name('Literate Haskell') == Language.find_by_alias('lhs')
        assert Language.find_by_name('Literate Haskell') == Language.find_by_alias('literate-haskell')
        assert Language.find_by_name('Objective-C') == Language.find_by_alias('objc')
        assert Language.find_by_name('OpenEdge ABL') == Language.find_by_alias('openedge')
        assert Language.find_by_name('OpenEdge ABL') == Language.find_by_alias('progress')
        assert Language.find_by_name('OpenEdge ABL') == Language.find_by_alias('abl')
        assert Language.find_by_name('Parrot Internal Representation') == Language.find_by_alias('pir')
        assert Language.find_by_name('PowerShell') == Language.find_by_alias('posh')
        assert Language.find_by_name('Puppet') == Language.find_by_alias('puppet')
        assert Language.find_by_name('Pure Data') == Language.find_by_alias('pure-data')
        assert Language.find_by_name('Raw token data') == Language.find_by_alias('raw')
        assert Language.find_by_name('Ruby') == Language.find_by_alias('rb')
        assert Language.find_by_name('Ruby') == Language.find_by_alias('ruby')
        assert Language.find_by_name('Scheme') == Language.find_by_alias('scheme')
        assert Language.find_by_name('Shell') == Language.find_by_alias('bash')
        assert Language.find_by_name('Shell') == Language.find_by_alias('sh')
        assert Language.find_by_name('Shell') == Language.find_by_alias('shell')
        assert Language.find_by_name('Shell') == Language.find_by_alias('zsh')
        assert Language.find_by_name('TeX') == Language.find_by_alias('tex')
        assert Language.find_by_name('TypeScript') == Language.find_by_alias('ts')
        assert Language.find_by_name('VimL') == Language.find_by_alias('vim')
        assert Language.find_by_name('VimL') == Language.find_by_alias('viml')
        assert Language.find_by_name('reStructuredText') == Language.find_by_alias('rst')
        assert Language.find_by_name('YAML') == Language.find_by_alias('yml')

    def test_groups(self):
        # Test a couple identity cases
        assert Language.find_by_name('Perl') == Language.find_by_name('Perl').group
        assert Language.find_by_name('Python') == Language.find_by_name('Python').group
        assert Language.find_by_name('Ruby') == Language.find_by_name('Ruby').group

        # Test a few special groups
        assert Language.find_by_name('Assembly') == Language.find_by_name('GAS').group
        assert Language.find_by_name('C') == Language.find_by_name('OpenCL').group
        assert Language.find_by_name('Haskell') == Language.find_by_name('Literate Haskell').group
        assert Language.find_by_name('Java') == Language.find_by_name('Java Server Pages').group
        assert Language.find_by_name('Python') == Language.find_by_name('Cython').group
        assert Language.find_by_name('Python') == Language.find_by_name('NumPy').group
        assert Language.find_by_name('Shell') == Language.find_by_name('Batchfile').group
        assert Language.find_by_name('Shell') == Language.find_by_name('Gentoo Ebuild').group
        assert Language.find_by_name('Shell') == Language.find_by_name('Gentoo Eclass').group
        assert Language.find_by_name('Shell') == Language.find_by_name('Tcsh').group

        # Ensure everyone has a group
        for language in Language.all():
            assert language.group, "%s has no group" % language


    def test_search_term(self):
        assert 'perl' == Language.find_by_name('Perl').search_term
        assert 'python' == Language.find_by_name('Python').search_term
        assert 'ruby' == Language.find_by_name('Ruby').search_term
        assert 'common-lisp' == Language.find_by_name('Common Lisp').search_term
        assert 'html+erb' == Language.find_by_name('HTML+ERB').search_term
        assert 'max/msp' == Language.find_by_name('Max').search_term
        assert 'puppet' == Language.find_by_name('Puppet').search_term
        assert 'pure-data' == Language.find_by_name('Pure Data').search_term

        assert 'aspx-vb' == Language.find_by_name('ASP').search_term
        assert 'as3' == Language.find_by_name('ActionScript').search_term
        assert 'nasm' == Language.find_by_name('Assembly').search_term
        assert 'bat' == Language.find_by_name('Batchfile').search_term
        assert 'csharp' == Language.find_by_name('C#').search_term
        assert 'cpp' == Language.find_by_name('C++').search_term
        assert 'cfm' == Language.find_by_name('ColdFusion').search_term
        assert 'dpatch' == Language.find_by_name('Darcs Patch').search_term
        assert 'ocaml' == Language.find_by_name('F#').search_term
        assert 'pot' == Language.find_by_name('Gettext Catalog').search_term
        assert 'irc' == Language.find_by_name('IRC log').search_term
        assert 'lhs' == Language.find_by_name('Literate Haskell').search_term
        assert 'ruby' == Language.find_by_name('Mirah').search_term
        assert 'raw' == Language.find_by_name('Raw token data').search_term
        assert 'bash' == Language.find_by_name('Shell').search_term
        assert 'vim' == Language.find_by_name('VimL').search_term
        assert 'jsp' == Language.find_by_name('Java Server Pages').search_term
        assert 'rst' == Language.find_by_name('reStructuredText').search_term

    def test_popular(self):
        assert Language.find_by_name('Ruby').is_popular
        assert Language.find_by_name('Perl').is_popular
        assert Language.find_by_name('Python').is_popular
        assert Language.find_by_name('Assembly').is_unpopular
        assert Language.find_by_name('Brainfuck').is_unpopular

    def test_programming(self):
        assert 'programming' == Language.find_by_name('JavaScript').type
        assert 'programming' == Language.find_by_name('Perl').type
        assert 'programming' == Language.find_by_name('PowerShell').type
        assert 'programming' == Language.find_by_name('Python').type
        assert 'programming' == Language.find_by_name('Ruby').type
        assert 'programming' == Language.find_by_name('TypeScript').type

    def test_markup(self):
        assert 'markup' == Language.find_by_name('HTML').type
        assert 'markup' == Language.find_by_name('YAML').type

    def test_other(self):
        assert None == Language.find_by_name('Brainfuck').type
        assert None == Language.find_by_name('Makefile').type

    def test_searchable(self):
        assert True == Language.find_by_name('Ruby').is_searchable
        assert False == Language.find_by_name('Gettext Catalog').is_searchable
        assert False == Language.find_by_name('SQL').is_searchable

    def test_find_by_name(self):
        assert Language.find_by_name('Ruby') == Language.name_index['Ruby']

    def test_find_all_by_name(self):
        for language in Language.all():
            assert language == Language.find_by_name(language.name)
            assert language == Language.name_index[language.name]

    def test_find_all_by_alias(self):
        for language in Language.all():
            for name in language.aliases:
                assert language == Language.find_by_alias(name)

    def test_find_by_filename(self):
        assert [Language.find_by_name('Shell')] == Language.find_by_filename('PKGBUILD')
        assert [Language.find_by_name('Ruby')] == Language.find_by_filename('foo.rb')
        assert [Language.find_by_name('Ruby')] == Language.find_by_filename('foo/bar.rb')
        assert [Language.find_by_name('Ruby')] == Language.find_by_filename('Rakefile')
        assert [Language.find_by_name('Ruby')] == Language.find_by_filename('PKGBUILD.rb')
        assert Language.find_by_name('ApacheConf') == Language.find_by_filename('httpd.conf')[0]
        assert [Language.find_by_name('ApacheConf')] == Language.find_by_filename('.htaccess')
        assert Language.find_by_name('Nginx') == Language.find_by_filename('nginx.conf')[0]
        assert ['C', 'C++', 'Objective-C'] == sorted(map(lambda l: l.name, Language.find_by_filename('foo.h')))
        assert [] == Language.find_by_filename('rb')
        assert [] == Language.find_by_filename('.rb')
        assert [] == Language.find_by_filename('.nkt')
        assert [Language.find_by_name('Shell')] == Language.find_by_filename('.bashrc')
        assert [Language.find_by_name('Shell')] == Language.find_by_filename('bash_profile')
        assert [Language.find_by_name('Shell')] == Language.find_by_filename('.zshrc')

    def test_find(self):
        assert 'Ruby' == Language.find_by_name('Ruby').name
        assert 'Ruby' == Language.find_by_name('ruby').name
        assert 'C++' == Language.find_by_name('C++').name
        assert 'C++' == Language.find_by_name('c++').name
        assert 'C++' == Language.find_by_name('cpp').name
        assert 'C#' == Language.find_by_name('C#').name
        assert 'C#' == Language.find_by_name('c#').name
        assert 'C#' == Language.find_by_name('csharp').name
        assert None == Language.find_by_name('defunkt')

    def test_name(self):
        assert 'Perl' == Language.name_index['Perl'].name
        assert 'Python' == Language.name_index['Python'].name
        assert 'Ruby' ==  Language.name_index['Ruby'].name

    def test_escaped_name(self):
        assert 'C', Language.find_by_name('C').escaped_name
        assert 'C%23', Language.find_by_name('C#').escaped_name
        assert 'C%2B%2B', Language.find_by_name('C++').escaped_name
        assert 'Objective-C', Language.find_by_name('Objective-C').escaped_name
        assert 'Common%20Lisp', Language.find_by_name('Common Lisp').escaped_name

    def test_error_without_name(self):
        self.assertRaises(KeyError, Language, {})

    def test_color(self):
        assert '#701516' == Language.find_by_name('Ruby').color
        assert '#3581ba' == Language.find_by_name('Python').color
        assert '#f15501' == Language.find_by_name('JavaScript').color
        assert '#31859c' == Language.find_by_name('TypeScript').color

    def test_ace_mode(self):
        assert 'c_cpp' == Language.find_by_name('C++').ace_mode
        assert 'coffee' == Language.find_by_name('CoffeeScript').ace_mode
        assert 'csharp' == Language.find_by_name('C#').ace_mode
        assert 'css' == Language.find_by_name('CSS').ace_mode
        assert 'javascript' == Language.find_by_name('JavaScript').ace_mode

    def test_ace_modes(self):
        assert Language.find_by_name('Ruby') in Language.ace_modes()
        assert Language.find_by_name('FORTRAN') not in Language.ace_modes()

    def test_wrap(self):
        assert False == Language.find_by_name('C').wrap
        assert True == Language.find_by_name('Markdown').wrap

    def test_extensions(self):
        assert '.pl' in Language.find_by_name('Perl').extensions
        assert '.py' in Language.find_by_name('Python').extensions
        assert '.rb' in Language.find_by_name('Ruby').extensions

    def test_primary_extension(self):
        assert '.pl' == Language.find_by_name('Perl').primary_extension
        assert '.py' == Language.find_by_name('Python').primary_extension
        assert '.rb' == Language.find_by_name('Ruby').primary_extension
        assert '.js' == Language.find_by_name('JavaScript').primary_extension
        assert '.coffee' == Language.find_by_name('CoffeeScript').primary_extension
        assert '.t' == Language.find_by_name('Turing').primary_extension
        assert '.ts' == Language.find_by_name('TypeScript').primary_extension

        # This is a nasty requirement, but theres some code in GitHub that
        # expects this. Really want to drop this.
        for language in Language.all():
          assert language.primary_extension, "%s has no primary extension" % language

    def test_eql(self):
        assert Language.find_by_name('Ruby') == Language.find_by_name('Ruby')
        assert Language.find_by_name('Ruby') != Language.find_by_name('Python')

    def test_colorize(self):
        assert colorize == Language.find_by_name('Ruby').colorize("def foo\n  'foo'\nend\n")


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from framework import LinguistTestBase, main
from libs.samples import Samples, DATA

class TestSamples(LinguistTestBase):

    def test_verify(self):
        data = DATA
        def _inject(d):
            return reduce(lambda x,y: x + y[1], d.items(), 0)
        assert data['languages_total'] == _inject(data['languages'])
        assert data['tokens_total'] == _inject(data['language_tokens'])
        assert data['tokens_total'] == reduce(lambda x, y: x + _inject(dict(y[1])), data['tokens'].items(), 0)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from framework import LinguistTestBase, main
from libs.md5 import MD5


class TestMD5(LinguistTestBase):

    def test_hexdigest_string(self):
        assert '51d5ab3bb6ceacfd28fb3435b44df469' == MD5.hexdigest('foo')
        assert 'ed7dd53c428168f872ff18e1a4e7a0cb' == MD5.hexdigest('bar')

    def test_hexdigest_integer(self):
        assert 'ae3b28cde02542f81acce8783245430d' == MD5.hexdigest(1)
        assert '23e7c6cacb8383f878ad093b0027d72b' == MD5.hexdigest(2)

    def test_hexdigest_boolean(self):
        assert 'e62f404a21c359f79b2fac2c7a433eaf' == MD5.hexdigest(True)
        assert '9fa6d44ce6c9c565572d7b5a89e8205f' == MD5.hexdigest(False)

        assert MD5.hexdigest("True") != MD5.hexdigest(True)
        assert MD5.hexdigest("False") != MD5.hexdigest(False)

    def test_hexdigest_none(self):
        assert 'a4179d01d58ec2f9c54faeb814a6a50c' == MD5.hexdigest(None)
        assert MD5.hexdigest("None") != MD5.hexdigest(None)

    def test_hexdigest_list(self):
        assert '10ae9fc7d453b0dd525d0edf2ede7961' == MD5.hexdigest([])
        assert '5b89237adcc067a06bdf636d70d15335' == MD5.hexdigest([1])
        assert '60bbd1bf2ba7b4d7e9306969d693422d' == MD5.hexdigest([1, 2, 3])
        assert '57a3796c2e2be53df52f0731054e2b9c' == MD5.hexdigest([1, 2, [3]])

    def test_hexdigest_dict(self):
        assert 'bb4c374392133719a324ab1ba2799cd6' == MD5.hexdigest({})
        assert '1de63be82bec8f13e58d5b2370846df1' == MD5.hexdigest({'a': 1})
        assert '27fbbe666112c19d5d30a3f39f433649' == MD5.hexdigest({'a': 1, 'b': 2})
        assert MD5.hexdigest({'a': 1, 'b': 2}) == MD5.hexdigest({'b': 2, 'a': 1})


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

import re
from framework import LinguistTestBase, main
from libs.strscan import StringScanner, get_regex, text_coords

class TestStringScanner(LinguistTestBase):

    def setUp(self):
        self.string = 'hello'
        self.strscan = StringScanner(self.string)
        super(TestStringScanner, self).setUp()

    def test_is_eos(self):
        assert self.strscan.is_eos == False
        self.strscan.terminate
        assert self.strscan.is_eos == True

    def test_getch(self):
        s = self.strscan
        pos = s.pos
        assert 'h' == s.getch
        assert s.pos == (pos + 1)
        s.getch
        assert 'l' == s.getch

    def test_peek(self):
        s = self.strscan
        r = s.peek(4)
        assert r == 'hell'

    def test_rest(self):
        s = self.strscan
        s.scan('hel')
        assert self.string == s.string

    def test_matched(self):
        s = self.strscan
        s.scan('hel')
        assert 'hel' == s.matched

    def test_pre_match(self):
        s = self.strscan
        assert 2 == s.skip('he')
        assert 'll' == s.scan('ll')
        assert 'he' == s.pre_match

    def test_post_match(self):
        s = self.strscan
        s.skip('he')
        s.scan('ll')
        assert 'o' == s.post_match

    def test_unscan(self):
        s = self.strscan
        s.skip('he')
        assert 'llo' == s.rest
        s.unscan
        assert 0 == s.pos
        assert self.string == s.string

    def test_is_beginning_of_line(self):
        s = self.strscan
        assert s.is_beginning_of_line == True

    def test_terminate(self):
        s = self.strscan
        s.getch
        s.terminate
        assert s.pos == len(self.string)
        assert s.match == None

    def test_scan_full(self):
        s = self.strscan
        assert None == s.scan_full("l")
        assert 'he' == s.scan_full("he")
        assert 2 == s.pos
        assert 'll' == s.scan_full('ll', advance_pointer=False)
        assert 2 == s.pos
        assert 2 == s.scan_full('ll', return_string=False, advance_pointer=False)
        assert 2 == s.pos

    def test_search_full(self):
        s = self.strscan
        assert 'he' == s.search_full('e')
        assert 2 == s.pos
        assert 'llo' == s.search_full('lo', advance_pointer=False)
        assert 2 == s.pos
        assert 3 == s.search_full('o', return_string=False, advance_pointer=False)

    def test_scan(self):
        s = self.strscan
        assert 0 == s.pos
        s.scan('world')
        s.scan('luo')
        assert 0 == s.pos
        assert 'hel' == s.scan('hel')
        assert 3 == s.pos

    def test_scan_until(self):
        s = self.strscan
        assert 'hel' == s.scan_until('el')
        assert 3 == s.pos 

    def test_scan_upto(self):
        s = self.strscan
        assert 'h' == s.scan('h')
        assert 'el' == s.scan_upto('lo')
        assert 3 == s.pos
        assert [0, 1, 3] == s.pos_history

    def test_skip(self):
        s = self.strscan
        assert 3 == s.skip('hel')

    def test_skip_unitl(self):
        s = self.strscan
        assert 3 == s.skip_until('l')

    def test_check(self):
        s = self.strscan
        assert 'hell' == s.check('hell')
        assert 0 == s.pos

    def test_check_until(self):
        s = self.strscan
        assert 'hell' == s.check('hell')
        assert 0 == s.pos

    def test_exists(self):
        s = self.strscan
        assert 3 == s.exists('l')
        assert 0 == s.pos

    def test_coords(self):
        s = StringScanner("abcdef\nghijkl\nmnopqr\nstuvwx\nyz")
        assert (0, 0, "abcdef") == s.coords
        s.pos += 4
        assert (0, 4, 'abcdef') == s.coords
        s.pos += 2
        assert (0, 6, 'abcdef') == s.coords
        s.pos += 1
        assert (1, 0, 'ghijkl') == s.coords
        s.pos += 8
        assert (2, 1, 'mnopqr') == s.coords

    def test_text_coords(self):
        s = "abcdef\nghijkl\nmnopqr\nstuvwx\nyz"
        assert (0, 0, 'abcdef') == text_coords(s, 0)
        assert (0, 4, 'abcdef') == text_coords(s, 4)
        assert (0, 6, 'abcdef') == text_coords(s, 6)
        assert (1, 0, 'ghijkl') == text_coords(s, 7)
        assert (1, 4, 'ghijkl') == text_coords(s, 11)
        assert (2, 1, 'mnopqr') == text_coords(s, 15)

    def test_get_regex(self):
        r1 = get_regex("x")
        assert isinstance(r1, re._pattern_type)

        r2 = get_regex(re.compile("x")) 
        assert isinstance(r2, re._pattern_type)

        self.assertRaises(TypeError, get_regex, 1)
        self.assertRaises(TypeError, get_regex, [])
        self.assertRaises(TypeError, get_regex, {})


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

import re
from framework import LinguistTestBase, main
from libs.generated import Generated, XCODE_PROJECT_EXT_NAMES

TEST_FILE = "../samples/%s"

class TestGenerated(LinguistTestBase):

    # def setUp(self):
    #     super(TestGenerated, self).setUp()

    # def generate(self, name, data):
    #     return Generated(name, data)

    def test_is_xcode_project_file(self):
        for ext in XCODE_PROJECT_EXT_NAMES:
            assert True == Generated('t'+ext, '').is_xcode_project_file
        assert False == Generated('t.py', '').is_xcode_project_file

    def test_is_minified_javascript(self):
        js_mini_name = 'jquery-1.6.1.min.js'
        js_mini_path = TEST_FILE % ('JavaScript/%s' % js_mini_name)
        js_mini_file = lambda : open(js_mini_path).read()
        g = Generated(js_mini_name, js_mini_file)
        assert True == g.is_minified_javascript

        g = Generated(js_mini_name, js_mini_file())
        assert True == g.is_minified_javascript

        js_name = 'jquery-1.6.1.js'
        js_path = TEST_FILE % ('JavaScript/%s' % js_name)
        js_file = lambda : open(js_path).read()
        g = Generated(js_name, js_file)
        assert False == g.is_minified_javascript


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

import os
import sys

TEST_DIR = os.path.dirname(os.path.realpath(__file__))
ROOT_DIR = os.path.dirname(TEST_DIR)
LIBS_DIR = os.path.join(ROOT_DIR, "linguist")
sys.path.insert(0, LIBS_DIR)

from unittest import main, TestCase


class LinguistTestBase(TestCase):
    def setUp(self):
        pass



# -*- coding: utf-8 -*-
import os
import re
from collections import defaultdict
from functools import partial

from file_blob import FileBlob

STARTS_WITH_DOT_REGEX = re.compile('\.[^/]')

class Repository(object):
    """
    A Repository is an abstraction of a Grit::Repo or a basic file
    system tree. It holds a list of paths pointing to Blobish objects.

    Its primary purpose is for gathering language statistics across
    the entire project.
    """

    def __init__(self, enum):
        """
        Public: Initialize a new Repository

        enum - Enumerator that responds to `each` and
            yields Blob objects

        Returns a Repository
        """
        self.enum = enum
        self.computed_stats = False
        self._language = self._size = None
        self.sizes = defaultdict(int)

    def __repr__(self):
        return '<Repository computed_stats:%s>' % self.computed_stats

    @classmethod
    def from_directory(cls, base_path):
        """
        Public: Initialize a new Repository from a File directory

        base_path - A path String

        Returns a Repository
        """
        blob = partial(FileBlob, base_path=base_path)
        enum = map(blob, cls.get_files(base_path))
        return cls(enum)

    @staticmethod
    def get_files(base_path):
        for root, dirs, files in os.walk(base_path):
            if STARTS_WITH_DOT_REGEX.search(root):
                continue
            for f in files:
                if f.startswith('.'):
                    continue
                yield os.path.join(root, f)

    @property
    def languages(self):
        """
        Public: Returns a breakdown of language stats.

          Examples

            # => { Language['Ruby'] => 46319,
                   Language['JavaScript'] => 258 }

          Returns a Hash of Language keys and Integer size values.
        """
        self.compute_stats
        return self.sizes

    @property
    def language(self):
        """
        Public: Get primary Language of repository.

        Returns a Language
        """
        self.compute_stats
        return self._language

    @property
    def size(self):
        """
        Public: Get the total size of the repository.

        Returns a byte size Integer
        """
        self.compute_stats
        return self._size

    @property
    def compute_stats(self):
        """
        Internal: Compute language breakdown for each blob in the Repository.

        Returns nothing
        """
        if self.computed_stats:
            return

        for blob in self.enum:
            # Skip files that are link
            if blob.is_link:
                continue

            # Skip files that are likely binary
            if blob.is_likely_binary:
                continue

            # Skip vendored or generated blobs
            if blob.is_vendored or blob.is_generated or blob.language is None:
                continue

            # Only include programming languages
            if blob.language.type == 'programming':
                self.sizes[blob.language.group] += blob.size

        # Compute total size
        self._size = reduce(lambda x, y: x+y[1], self.sizes.items(), 0)

        # Get primary language
        primary = sorted(self.sizes.items(), key=lambda t: t[1], reverse=True)
        if primary:
            self._language = primary[0][0]

        self.computed_stats = True



# -*- coding: utf-8 -*-

from pygments.lexers import find_lexer_class
from framework import LinguistTestBase, main, ROOT_DIR
from libs.repository import Repository
from libs.language import Language

class TestRepository(LinguistTestBase):

    def repo(self, base_path):
        return Repository.from_directory(base_path)

    def linguist_repo(self):
        return self.repo(ROOT_DIR)

    def test_linguist_language(self):
        assert self.linguist_repo().language == Language.find_by_name('Python')

    def test_linguist_languages(self):
        assert self.linguist_repo().languages[Language.find_by_name('Python')] > 2000

    def test_linguist_size(self):
        assert self.linguist_repo().size > 3000

    def test_binary_override(self):
        assert self.repo(ROOT_DIR + '/samples/Nimrod').language == Language.find_by_name('Nimrod')


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

import hashlib

class MD5(object):

    def __repr__(self):
        return '<MD5>'

    @classmethod
    def hexdigest(cls, obj):
        digest = hashlib.md5()

        if isinstance(obj, (str, int)):
            digest.update(obj.__class__.__name__)
            digest.update('%s' % obj)

        elif isinstance(obj, bool) or obj is None:
            digest.update(obj.__class__.__name__)

        elif isinstance(obj, (list, tuple)):
            digest.update(obj.__class__.__name__)
            for e in obj:
                digest.update(cls.hexdigest(e))

        elif isinstance(obj, dict):
            digest.update(obj.__class__.__name__)
            hexs = [cls.hexdigest([k, v]) for k, v in obj.iteritems()]
            hexs.sort()
            for e in hexs:
                digest.update(e)

        else:
            raise TypeError, "can't convert %s into String" % obj

        return digest.hexdigest()



# -*- coding: utf-8 -*-
import json
from os import listdir
from os.path import realpath, dirname, exists, join, splitext
from classifier import Classifier
from md5 import MD5

DIR = dirname(realpath(__file__))
ROOT = join(dirname(dirname(DIR)), "samples")
PATH = join(DIR, "samples.json")
DATA = {}

if exists(PATH):
    DATA = json.load(open(PATH))

class Samples(object):
    """
    Model for accessing classifier training data.
    """

    def __repr__(self):
        return '<Samples>'

    @classmethod
    def generate(self):
        data = self.data()
        json.dump(data, open(PATH, 'w'), indent=2)

    @classmethod
    def each(cls, func):
        for category in listdir(ROOT):
            if category in ('Binary', 'Text'):
                continue
            dirname = join(ROOT, category)
            for filename in listdir(dirname):
                if filename == 'filenames':
                    subdirname = join(dirname, filename)
                    for subfilename in listdir(subdirname):
                        func({'path': join(subdirname, subfilename),
                                'language': category,
                                'filename': subfilename})
                else:
                    _extname = splitext(filename)[1]
                    if _extname == '':
                        raise '%s is missing an extension, maybe it belongs in filenames/subdir' % (join(dirname, filename))
                    path = join(dirname, filename)
                    func({'path': join(dirname, filename),
                            'language': category,
                            'extname': _extname})


    @classmethod
    def data(cls):
        """
        Public: Build Classifier from all samples.

        Returns trained Classifier.
        """
        db = {}
        db['extnames'] = {}
        db['filenames'] = {}

        def _learn(sample):
            _extname = sample.get('extname')
            _filename = sample.get('filename')
            _langname = sample['language']

            if _extname:
                db['extnames'][_langname] = db['extnames'].get(_langname, [])
                if _extname not in db['extnames'][_langname]:
                    db['extnames'][_langname].append(_extname)
                    db['extnames'][_langname].sort()

            if _filename:
                db['filenames'][_langname] = db['filenames'].get(_langname, [])
                db['filenames'][_langname].append(_filename)
                db['filenames'][_langname].sort()

            data = open(sample['path']).read()
            Classifier.train(db, _langname, data)

        cls.each(_learn)

        db['md5'] = MD5.hexdigest(db)
        return db



# -*- coding: utf-8 -*-

import re
import urllib
from os.path import realpath, dirname, basename, splitext, join
from collections import defaultdict

import yaml
from pygments import lexers
from pygments import highlight
from pygments.formatters import HtmlFormatter

from classifier import Classifier
from samples import Samples, DATA

DIR = dirname(realpath(__file__))
POPULAR_PATH = join(DIR, "popular.yml")
LANGUAGES_PATH = join(DIR, "languages.yml")

POPULAR = yaml.load(open(POPULAR_PATH))
LANGUAGES = yaml.load(open(LANGUAGES_PATH))

class Language(object):
    """
    Language names that are recognizable by GitHub. Defined languages
    can be highlighted, searched and listed under the Top Languages page.

    Languages are defined in `lib/linguist/languages.yml`.
    """

    languages       = []
    index           = {}
    name_index      = {}
    alias_index     = {}
    extension_index = defaultdict(list)
    filename_index = defaultdict(list)

    _colors = []
    _ace_modes = []

    # Valid Languages types
    TYPES = ('data', 'markup', 'programming')

    @classmethod
    def create(cls, attributes={}):
        language = cls(attributes)
        cls.languages.append(language)

        # All Language names should be unique. Raise if there is a duplicate.
        if cls.name_index.has_key(language.name):
            raise ValueError, "Duplicate language name: %s" % language.name
        # Language name index
        name = language.name
        cls.index[name] = cls.name_index[name] = language
        cls.index[name.lower()] = cls.name_index[name.lower()] = language

        # All Language aliases should be unique.
        # Raise if there is a duplicate.
        for name in language.aliases:
            if cls.alias_index.has_key(name):
                raise ValueError, "Duplicate alias: %s " % name
            cls.index[name] = cls.alias_index[name] = language

        for extension in language.extensions:
            if not extension.startswith('.'):
                raise ValueError, "Extension is missing a '.': %s"%extension
            cls.extension_index[extension].append(language)

        for filename in language.filenames:
            cls.filename_index[filename].append(language)

        return language


    def __init__(self, attributes={}):
        # name is required
        if 'name' not in attributes.keys():
            raise KeyError('missing name')
        self.name = attributes['name']

        # Set type
        self.type = attributes.get('type')
        if self.type and self.type not in self.TYPES:
            raise ValueError('invalid type: %s' %self.type)

        self.color = attributes['color']

        # Set aliases
        aliases = attributes.get('aliases') or []
        self.aliases = [self.default_alias_name] + aliases

        # Lookup Lexer object
        lexer = attributes.get('lexer') or self.name
        self.lexer = lexers.find_lexer_class(lexer)
        if not self.lexer:
            raise TypeError('%s is missing lexer' % self.name)

        self.ace_mode = attributes['ace_mode']
        self.wrap = attributes.get('wrap') or False

        # Set legacy search term
        self.search_term = attributes.get('search_term') or self.default_alias_name

        # Set extensions or default to [].
        self.extensions = attributes.get('extensions') or []
        self.filenames = attributes.get('filenames') or []

        self.primary_extension = attributes.get('primary_extension')
        if not self.primary_extension:
            raise KeyError('%s is missing primary extension' % self.name)

        # Prepend primary extension unless its already included
        if self.primary_extension not in self.extensions:
            self.extensions = [self.primary_extension] + self.extensions

        # Set popular, and searchable flags
        self.popular = attributes['popular'] if attributes.has_key('popular') else False
        self.searchable = attributes['searchable'] if attributes.has_key('searchable') else True

        # If group name is set, save the name so we can lazy load it later
        group_name = attributes.get('group_name')
        if group_name:
            self._group = None
            self.group_name = group_name
        else:
            self._group = self

    def __repr__(self):
        return '<Language name:%s>' % self.name

    def __eq__(self, target):
        return self.name == target.name and self.extensions == target.extensions

    @classmethod
    def find_by_name(cls, name):
        """
        Public: Look up Language by its proper name.

        name - The String name of the Language

         Examples

           Language.find_by_name('Ruby')
           # => #<Language name:"Ruby">

        Returns the Language or nil if none was found.
        """
        return cls.name_index.get(name) or cls.find_by_alias(name)

    @classmethod
    def find_by_filename(cls, filename):
        """
        Public: Look up Languages by filename.
        filename - The path String.

        Examples
          Language.find_by_filename('foo.rb')
          # => [#<Language name:"Ruby">]

        Returns all matching Languages or [] if none were found.
        """
        name, extname = basename(filename), splitext(filename)[1]
        return cls.filename_index.get(name,[]) + cls.extension_index.get(extname,[])

    @classmethod
    def find_by_alias(cls, name):
        """
        Public: Look up Language by one of its aliases.

        name - A String alias of the Language

        Examples

          Language.find_by_alias('cpp')
          # => #<Language name:"Ruby">

        Returns the Language or nil if none was found.
        """
        return cls.alias_index.get(name)

    @classmethod
    def colors(cls):
        if cls._colors:
            return cls._colors
        cls._colors = sorted(filter(lambda l: l.color, cls.all()), key=lambda l: l.name.lower())
        return cls._colors

    @classmethod
    def ace_modes(cls):
        if cls._ace_modes:
            return cls._ace_modes
        cls._ace_modes = sorted(filter(lambda l: l.ace_mode, cls.all()), key=lambda l: l.name.lower())
        return cls._ace_modes


    @classmethod
    def all(cls):
        """
        Public: Get all Languages
        Returns an Array of Languages
        """
        return cls.languages


    @classmethod
    def detect(cls, name, data, mode=None):
        """
        Public: Detects the Language of the blob.

          name - String filename
          data - String blob data. A block also maybe passed in for lazy
                 loading. This behavior is deprecated and you should
                 always pass in a String.
          mode - Optional String mode (defaults to nil)

        Returns Language or nil.

        A bit of an elegant hack. If the file is executable but
        extensionless, append a "magic" extension so it can be
        classified with other languages that have shebang scripts.
        """
        extname = splitext(name)[1]
        if not extname and mode and (int(mode, 8)&05 == 05):
            name += ".script!"

        possible_languages = cls.find_by_filename(name)

        if len(possible_languages) == 1:
            return possible_languages[0]

        data = data() if callable(data) else data
        if data is None or data == "":
            return

        _pns = [p.name for p in possible_languages]
        result = Classifier.classify(DATA, data, _pns)
        result = result and result[0][0]
        if result:
            return cls.find_by_name(result)

    def colorize(self, text, options={}):
        return highlight(text, self.lexer(), HtmlFormatter(**options))

    @property
    def group(self):
        return self._group or self.find_by_name(self.group_name)

    @property
    def is_popular(self):
        """
        Is it popular?
        Returns true or false
        """
        return self.popular

    @property
    def is_unpopular(self):
        """
        Is it not popular?
        Returns true or false
        """
        return not self.popular

    @property
    def is_searchable(self):
        """
        Is it searchable?

        Unsearchable languages won't by indexed by solr and won't show
        up in the code search dropdown.

        Returns true or false
        """
        return self.searchable


    @property
    def default_alias_name(self):
        """
        Internal: Get default alias name
        Returns the alias name String
        """
        return re.sub('\s', '-', self.name.lower())

    @property
    def escaped_name(self):
        """
        Public: Get URL escaped name.

        Examples:
          "C%23"
          "C%2B%2B"
          "Common%20Lisp"

        Returns the escaped String.
        """
        return urllib.quote(self.name, '')

extensions = DATA['extnames']
filenames = DATA['filenames']
popular = POPULAR

for name, options in LANGUAGES.items():
    options['extensions'] = options.get('extensions', [])
    options['filenames'] = options.get('filenames', [])

    def _merge(data, item_name):
        items = data.get(name)
        if not items:
            return

        for item in items:
            if item in options[item_name]:
                Warning('%s %s is already defined in samples/. Remove from languages.yml.' % (name, item))
            else:
                options[item_name].append(item)
    _merge(extensions, 'extensions')
    _merge(filenames, 'filenames')

    Language.create(dict(
        name=name,
        color=options.get('color'),
        type=options.get('type'),
        aliases=options.get('aliases', []),
        lexer=options.get('lexer'),
        ace_mode=options.get('ace_mode'),
        wrap=options.get('wrap'),
        group_name=options.get('group'),
        searchable=options.get('searchable', True),
        search_term=options.get('search_term'),
        extensions=sorted(options['extensions']),
        primary_extension=options.get('primary_extension'),
        filenames=options['filenames'],
        popular=name in popular
    ))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name = 'linguist',
    version = '0.0.1',
    keywords = ('linguist',),
    description = 'Language Savant',
    long_description = 'Language Savant',
    license = 'MIT License',

    url = 'https://github.com/liluo/linguist',
    author = 'liluo',
    author_email = 'i@liluo.org',

    packages = find_packages(),
    include_package_data = True,
    platforms = 'any',
    install_requires = ['PyYAML', 'pygments>=1.6', 'charlockholmes'],
    classifiers = [],
)



# -*- coding: utf-8 -*-

import re
import urllib
import mimetypes
from os.path import realpath, dirname, splitext, basename, join, islink

import yaml
import charlockholmes
from pygments import lexers
from pygments import highlight
from pygments.formatters import HtmlFormatter

from language import Language
from generated import Generated

MEGABYTE = 1024 * 1024

DIR = dirname(realpath(__file__))
VENDOR_PATH = join(DIR, "vendor.yml")
VENDORED_PATHS = yaml.load(open(VENDOR_PATH))
VENDORED_REGEXP = re.compile('|'.join(VENDORED_PATHS))

# TODO python mimetypes 很不全
if not mimetypes.inited:
    mimetypes.init()
    mimetypes.add_type('application/x-ruby', '.rb')
    mimetypes.add_type('application/x-python', '.py')


class BlobHelper(object):
    """
    BlobHelper is a mixin for Blobish classes that respond to "name",
      "data" and "size" such as Grit::Blob.
    """

    @property
    def ext_name(self):
        """
        Public: Get the extname of the path

        Examples
          blob(name='foo.rb').extname
          # => '.rb'

        Returns a String
        """
        return splitext(self.name)[1].lower()

    @property
    def _mime_type(self):
        if hasattr(self, '__mime_type'):
            return self.__mime_type
        _type, _encoding = mimetypes.guess_type(self.name)
        self.__mime_encodeing = _encoding
        self.__mime_type = _type
        return _type

    @property
    def mime_type(self):
        """
        Public: Get the actual blob mime type

        Examples

          # => 'text/plain'
          # => 'text/html'

        Returns a mime type String.
        """
        return self._mime_type or 'text/plain'

    @property
    def is_binary_mime_type(self):
        """
        Internal: Is the blob binary according to its mime type

        Return true or false
        """
        self._mime_type
        return bool(self.__mime_encodeing)

    @property
    def is_likely_binary(self):
        """
        Internal: Is the blob binary according to its mime type,
        overriding it if we have better data from the languages.yml
        database.

        Return true or false
        """
        return self.is_binary_mime_type and not Language.find_by_filename(self.name)

    @property
    def content_type(self):
        """
        Public: Get the Content-Type header value

        This value is used when serving raw blobs.

        Examples

          # => 'text/plain; charset=utf-8'
          # => 'application/octet-stream'

        Returns a content type String.
        """
        if hasattr(self, '_content_type'):
            return self._content_type

        if self.is_binary_mime_type or self.is_binary:
            self._content_type = self.mime_type
        else:
            encoding = self.encoding
            self._content_type = "text/plain; charset=%s" % encoding.lower() if encoding else "text/plain"
        return self._content_type

    @property
    def disposition(self):
        """
        Public: Get the Content-Disposition header value

        This value is used when serving raw blobs.

          # => "attachment; filename=file.tar"
          # => "inline"

        Returns a content disposition String.
        """
        if self.is_text or self.is_image:
            return 'inline'
        elif self.name is None:
            return 'attachment'
        else:
            return 'attachment; filename=%s' % urllib.quote_plus(basename(self.name))

    @property
    def encoding(self):
        if self.detect_encoding:
            return self.detect_encoding.get('encoding')

    @property
    def detect_encoding(self):
        """
        Try to guess the encoding

        Returns: a hash, with :encoding, :confidence, :type
                 this will return nil if an error occurred during detection or
                 no valid encoding could be found
        """
        if hasattr(self, '_detect_encoding'):
            return self._detect_encoding

        if self.data:
            self._detect_encoding = charlockholmes.detect(self.data)
        return self._detect_encoding

    @property
    def is_image(self):
        """
        Public: Is the blob a supported image format?

        Return true or false
        """
        return self.ext_name in ('.png', '.jpg', '.jpeg', '.gif')

    @property
    def is_solid(self):
        """
        Public: Is the blob a support 3D model format?

        Return true or false
        """
        return self.ext_name in ('.stl', '.obj')

    @property
    def is_pdf(self):
        """
        Public: Is the blob a supported 3D model format?

        Return true or false
        """
        return self.ext_name == '.pdf'

    @property
    def is_text(self):
        """
        Public: Is the blob text?

        Return true or false
        """
        return not self.is_binary

    @property
    def is_binary(self):
        if self.data is None:
            # Large blobs aren't even loaded into memory
            return True
        elif self.data == "":
            # Treat blank files as text
            return False
        elif self.encoding is None:
            # Charlock doesn't know what to think
            return True
        else:
            # If Charlock says its binary
            return self.detect_encoding.get('type') == 'binary'

    @property
    def is_large(self):
        """
        Public: Is the blob too big to load?

        Return false or true
        """
        return self.size > MEGABYTE

    @property
    def loc(self):
        """
        Public: Get number of lines of code

        Requires Blob#data

        Returns Integer
        """
        return len(self.lines)

    @property
    def sloc(self):
        """
        Public: Get number of source lines of code

        Requires Blob#data

        Returns Integer
        """
        return len(filter(re.compile('\S').match, self.lines))

    @property
    def is_safe_to_colorize(self):
        """
        Public: Is the blob safe to colorize?

        We use Pygments.rb for syntax highlighting blobs, which
        has some quirks and also is essentially 'un-killable' via
        normal timeout.  To workaround this we try to
        carefully handling Pygments.rb anything it can't handle.

        Return true or false
        """
        return not self.is_large and self.is_text and not self.is_high_ratio_of_long_lines

    @property
    def is_high_ratio_of_long_lines(self):
        """
        Internal: Does the blob have a ratio of long lines?
        These types of files are usually going to make Pygments.rb
        angry if we try to colorize them.

        Return true or false
        """
        if self.loc == 0:
            return False
        return self.size / self.loc > 5000

    @property
    def is_viewable(self):
        """
        Public: Is the blob viewable?

        Non-viewable blobs will just show a "View Raw" link

        Return true or false
        """
        return not self.is_large and self.is_text

    @property
    def is_vendored(self):
        """
        Public: Is the blob in a vendored directory?

        Vendored files are ignored by language statistics.

        See "vendor.yml" for a list of vendored conventions that match
        this pattern.

        Return true or false
        """
        if VENDORED_REGEXP.search(self.name):
            return True

    @property
    def lines(self):
        """
        Public: Get each line of data

        Requires Blob#data

        Returns an Array of lines
        """
        if hasattr(self, '_lines'):
            return self._lines
        if self.is_viewable and self.data:
            self._lines = self.data.split(self.line_split_character, -1)
        else:
            self._lines = []
        return self._lines

    @property
    def line_split_character(self):
        """
        Character used to split lines. This is almost always "\n" except when Mac
        Format is detected in which case it's "\r".

        Returns a split pattern string.
        """
        if hasattr(self, '_line_split_character'):
            return self._line_split_character
        if self.is_mac_format:
            self._line_split_character = '\r'
        else:
            self._line_split_character = '\n'
        return self._line_split_character

    @property
    def is_mac_format(self):
        """
        Public: Is the data in ** Mac Format **. This format uses \r (0x0d) characters
        for line ends and does not include a \n (0x0a).

        Returns true when mac format is detected.
        """
        if not self.is_viewable:
            return

        data = self.data[0:4096]
        if '\r' in data:
            pos = data.index('\r')
            return data[pos + 1] != '\n'

    @property
    def is_generated(self):
        """
        Public: Is the blob a generated file?

        Generated source code is suppressed in diffs and is ignored by
        language statistics.

        May load Blob#data

        Return true or false
        """
        if hasattr(self, '_is_generated'):
            return self._is_generated

        def _data_func():
            return self.data

        self._is_generated = Generated.is_generated(self.name, _data_func)
        return self._is_generated

    @property
    def is_indexable(self):
        """
        Public: Should the blob be indexed for searching?

        Excluded:
          - Files over 0.1MB
          - Non-text files
          - Languages marked as not searchable
          - Generated source files

        Please add additional test coverage to
        `test/test_blob.rb#test_indexable` if you make any changes.

        Return true or false
        """
        if self.size > 100 * 1024:
            return False
        elif self.is_binary:
            return False
        elif self.ext_name == '.txt':
            return True
        elif self.language is None:
            return False
        elif not self.language.is_searchable:
            return False
        elif self.is_generated:
            return False
        else:
            return False

    @property
    def is_link(self):
        return islink(self.path)

    @property
    def language(self):
        """
        Public: Detects the Language of the blob.

        May load Blob#data

        Returns a Language or nil if none is detected
        """
        if hasattr(self, '_language'):
            return self._language

        def data():
            if self.is_binary_mime_type or self.is_binary:
                return ''
            return self.data
        self._language = Language.detect(self.name, data, self.mode)
        return self._language

    @property
    def lexer(self):
        """
        Internal: Get the lexer of the blob.

        Returns a Lexer.
        """
        return self.language.lexer if self.language else lexers.find_lexer_class('Text only')

    def colorize(self, options={}):
        """
        Public: Highlight syntax of blob

        options - A Hash of options (defaults to {})

        Returns html String
        """
        if not self.is_safe_to_colorize:
            return
        return highlight(self.data, self.lexer(), HtmlFormatter(**options))

    def colorize_without_wrapper(self, options={}):
        """
        Public: Highlight syntax of blob without the outer highlight div
        """
        text = self.colorize(options)
        if text:
            ret = re.compile(r'<div class="highlight"><pre>(.*?)<\/pre>\s*<\/div>', re.DOTALL).match(text)
            if ret:
                ret = ret.group(1)
        else:
            ret = ''
        return ret
